﻿using Proto;
using Proto.Cluster;

namespace TestProject;

/// <summary>
/// 액터 시스템 클러스터 호스트 서비스
/// </summary>
public class ActorSystemClusterHostedService : IHostedService
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 액터 시스템
    /// </summary>
    private readonly ActorSystem actorSystem;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - ActorSystemClusterHostedService(actorSystem)

    /// <summary>
    /// 생성자
    /// </summary>
    /// <param name="actorSystem">액터 시스템</param>
    public ActorSystemClusterHostedService(ActorSystem actorSystem)
    {
        this.actorSystem = actorSystem;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 시작하기 (비동기) - StartAsync(cancellationToken)

    /// <summary>
    /// 시작하기 (비동기)
    /// </summary>
    /// <param name="cancellationToken">취소 토큰</param>
    /// <returns>태스크</returns>
    public async Task StartAsync(CancellationToken cancellationToken)
    {
        Console.WriteLine("클러스터 멤버를 시작한다.");

        await this.actorSystem
            .Cluster()
            .StartMemberAsync();
    }

    #endregion
    #region 중단하기 (비동기) - StopAsync(cancellationToken)

    /// <summary>
    /// 중단하기 (비동기)
    /// </summary>
    /// <param name="cancellationToken">취소 토큰</param>
    /// <returns>태스크</returns>
    public async Task StopAsync(CancellationToken cancellationToken)
    {
        Console.WriteLine("클러스터 멤버를 중단한다.");

        await this.actorSystem
            .Cluster()
            .ShutdownAsync();
    }

    #endregion
}